/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bezahlcode;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.SystemUtils;

import de.frankmuenster.jameica.qrcodehandler.QrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.HibiscusAddress;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface {@link de.frankmuenster.jameica.qrcodehandler.QrCodeHandler} mit einem Hanlder fr
 * Bezahlcode Kontak_V2 (SEPA)
 *
 * @author Frank Mnster
 */
public class BezahlcodeContactV2Handler implements QrCodeHandler {

    /**
     * Name des Handlers
     */
    private static final String HANDLER_NAME = "BezahlCode - Authority:contact and contact_v2"; //$NON-NLS-1$

    /**
     * Handler ID fr contact v1
     */
    private static final String HANDLER_ID_1 = "bank://contact?"; //$NON-NLS-1$

    /**
     * Handler ID fr contact v2
     */
    private static final String HANDLER_ID_2 = "bank://contact_v2?"; //$NON-NLS-1$
    /**
     * Handler registrieren
     */
    static {
        QrCodeHandlerRegsistry.register(new BezahlcodeContactV2Handler());
    }

    /**
     * Standardkonstruktor
     */
    public BezahlcodeContactV2Handler() {
        // Absichtlich leer
    }

    /**
     * Liest den QR-Code und erstellt den Kontakt.
     *
     * @param qrCode
     *            der QR-Code
     * @throws QrCodeHandlerException
     */
    @Override
    public void handleQrCode(final String qrCode) throws QrCodeHandlerException {
        if (!qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_1)
                && !qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_2)) {
            Logger.error(I18N.tr("Bezahlcode.wrongQrCode", //$NON-NLS-1$
                    BezahlcodeContactV2Handler.HANDLER_ID_1) + I18N.tr("Bezahlcode.wrongQrCode2" //$NON-NLS-1$
                            , BezahlcodeContactV2Handler.HANDLER_ID_2));
            Logger.debug("QR-Code: " + qrCode); //$NON-NLS-1$
            throw new QrCodeHandlerException(I18N.tr("Bezahlcode.invalidQrCode")); //$NON-NLS-1$
        }
        if (Settings.isHibiscusInstalled()) {
            try {
                final String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1"); //$NON-NLS-1$
                final String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode.indexOf('?') + 1);
                final Map<String, String> params = BezahlCodeCommenFunctions.getQueryMap(qrCodeQuery);
                final HibiscusAddress address = this.getAdress(params);
                if (address != null) {
                    // View neu zeichnen
                    GUI.startView(de.willuhn.jameica.hbci.gui.views.EmpfaengerNew.class, address);
                }
            } catch (final UnsupportedEncodingException e1) {
                throw new QrCodeHandlerException(I18N.tr("Bezahlcode.invalidQrCodeExceptionText"), e1); //$NON-NLS-1$
            } catch (final RemoteException e) {
                throw new QrCodeHandlerException(I18N.tr("Bezahlcode.errorCreatingAddressObject"), e); //$NON-NLS-1$
            }
        } else {
            final String text = I18N.tr("BezahlcodeContactV2Handler.hibiscusNotInstalledLine1") //$NON-NLS-1$
                    + I18N.tr("Bezahlcode.hibiscusNotInstalledLine2"); //$NON-NLS-1$
            Logger.warn(text);
            try {
                Application.getCallback().notifyUser(text);
            } catch (final Exception e) { // NOPMD
                Logger.error(I18N.tr("Bezahlcode.errorOpenDialog") + e.getMessage()); //$NON-NLS-1$
            }
        }
    }

    /**
     * @param params
     * @return ein {@link HibiscusAddress} Objekt mit den Kontaktdaten
     * @throws RemoteException
     */
    @SuppressWarnings({ "PMD.NPathComplexity", "PMD.ModifiedCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    private HibiscusAddress getAdress(final Map<String, String> params) throws RemoteException {
        final HibiscusAddress address = (HibiscusAddress) de.willuhn.jameica.hbci.Settings.getDBService()
                .createObject(HibiscusAddress.class, null);
        // Name
        String value;
        if ((value = params.get("name")) != null) { //$NON-NLS-1$
            if (value.equals(value.toUpperCase(Locale.getDefault()))) {
                try {
                    if (Application.getCallback().askUser(BezahlCodeCommenFunctions.QUESTION_TEXT,
                            new String[] { value })) {
                        value = BezahlCodeCommenFunctions.formatString(value);
                    }
                } catch (final Exception e) { // NOPMD
                    Logger.error(I18N.tr("BezahlcodeSinglePayment.errorAskUser") + e.getMessage()); //$NON-NLS-1$
                }
            }
            address.setName(value);
        }
        if ((value = params.get("account")) != null) { //$NON-NLS-1$
            address.setKontonummer(value);
        }
        if ((value = params.get("bnc")) != null) { //$NON-NLS-1$
            address.setBlz(value);
        }
        if ((value = params.get("iban")) != null) { //$NON-NLS-1$
            address.setIban(value);
        }
        if ((value = params.get("bic")) != null) { //$NON-NLS-1$
            address.setBic(value);
        }
        if ((value = params.get("reason")) == null) { //$NON-NLS-1$
            // Test for reasonN
            final StringBuilder reasonN = new StringBuilder();
            if ((value = params.get("reason1")) != null) { //$NON-NLS-1$
                reasonN.append(value);
            }
            if ((value = params.get("reason2")) != null) { //$NON-NLS-1$
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            if ((value = params.get("reason3")) != null) { //$NON-NLS-1$
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            if ((value = params.get("reason4")) != null) { //$NON-NLS-1$
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            address.setKommentar(reasonN.toString());
        } else {
            address.setKommentar(value);
        }
        return address;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.QrCodeHandler#canHandle(java.lang.String)
     */
    @Override
    public boolean canHandle(final String qrCode) {
        return qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_1)
                || qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_2);
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.QrCodeHandler#getHandlerName()
     */
    @Override
    public String getHandlerName() {
        return BezahlcodeContactV2Handler.HANDLER_NAME;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.QrCodeHandler#isUeberweisung()
     */
    @Override
    public boolean isUeberweisung() {
        return false;
    }
}
